//--------------------------------------------------------------------
// TUCN, Computer Science Department
// Input/Output Systems and Peripheral Devices
//--------------------------------------------------------------------
// http://users.utcluj.ro/~baruch/en/pages/teaching/inputoutput-systems/laboratory.php
//--------------------------------------------------------------------
// File:		TestPort-e.cpp
// Date:		17.08.2005
// Modified:	25.02.2009
//--------------------------------------------------------------------
// Testing the parallel ports with base addresses 378h and 278h
//--------------------------------------------------------------------
// - Application example to illustrate port accesses
// - The PortTalk driver is used for port accesses
//--------------------------------------------------------------------
// - Win32 API functions used:
//   GetTextMetrics, TextOut
//--------------------------------------------------------------------
// - Operating system: Microsoft Windows 2000/XP
// - Compiler: Microsoft Visual Studio 2008
//--------------------------------------------------------------------

#include <windows.h>
#include "PortTalk_IOCTL.c"

//--------------------------------------------------------------------
// Function TestPort
//--------------------------------------------------------------------
//
// Function:	Tests the existence of parallel ports with base
//				addresses 378h and 278h
//
// Parameters:	hWnd - Handle to the application window
//				hDc  - Handle to the context of the display device
//
// Returns:		-
//
//--------------------------------------------------------------------

void TestPort (HWND hWnd, HDC hDc)
{
	TEXTMETRIC tm;				// structure with the display metrics
	long cLine;					// height of a text line
	WORD wPort;					// port address
	BYTE byVal;					// value read from port
	char szBuffer [50];			// buffer for display
	int nLung;
	int i;
	BOOL bSucc;

	char szMes1 [] = "Testing the parallel ports with the PortTalk driver";
	char szMes2 [] = "Parallel port with base address %03Xh";
	char szMes3 [] = "Value written to data register: AAh";
	char szMes4 [] = "Value written to data register: 55h";
	char szMes5 [] = "Value read from data register: %02Xh";
	char szMes6 [] = "The port exists and works correctly";
	char szMes7 [] = "The port does not exist or does not work correctly";

	GetTextMetrics (hDc, &tm);	// determines the display metrics
	cLine = tm.tmHeight + tm.tmExternalLeading;

	OpenPortTalk ();			// initialize the PortTalk driver

// Display the start message
	TextOut (hDc, 10, 1*cLine, szMes1, strlen (szMes1));

// Test the parallel ports and display the messages
	wPort = 0x378;				// base address of the first port
	for (i = 0, bSucc = FALSE; i < 2; i++) {
		nLung = wsprintf (szBuffer, szMes2, wPort);
		TextOut (hDc, 10, (3+i*8)*cLine, szBuffer, nLung);
		outportb (wPort, 0xAA);	// write the value to the data register
		byVal = inportb (wPort);// read the value written
		TextOut (hDc, 10, (5+i*8)*cLine, szMes3, strlen (szMes3));
		nLung = wsprintf (szBuffer, szMes5, byVal);
		TextOut (hDc, 10, (6+i*8)*cLine, szBuffer, nLung);
		if (byVal == 0xAA)
			bSucc = TRUE;
		outportb (wPort, 0x55);	// write the value to the data register
		byVal = inportb (wPort);// read the value written
		TextOut (hDc, 10, (7+i*8)*cLine, szMes4, strlen (szMes4));
		nLung = wsprintf (szBuffer, szMes5, byVal);
		TextOut (hDc, 10, (8+i*8)*cLine, szBuffer, nLung);
		if (bSucc && (byVal == 0x55))
			TextOut (hDc, 10, (9+i*8)*cLine, szMes6, strlen (szMes6));
		else
			TextOut (hDc, 10, (9+i*8)*cLine, szMes7, strlen (szMes7));
		wPort = 0x278;			// base address of the second port
	}

	ClosePortTalk ();
}